@@ -33,30 +33,30 @@ use NeedRestart qw(:ui);
 
 needrestart_ui_register(__PACKAGE__, NEEDRESTART_PRIO_LOW);
 
-sub progress_prep($$$) {
+
+sub _announce {
     my $self = shift;
-    my ($max, $out) = @_;
+    my $message = shift;
+    my %vars = @_;
 
-    $self->SUPER::progress_prep($max, $out);
-
-    $|++;
-
-    print "$out";
+    print "Pending kernel upgrade!\n\nRunning kernel version:\n  $vars{KVERSION}\n\nDiagnostics:\n  $message\n\nRestarting the system to load the new kernel will not be handled automatically, so you should consider rebooting. [Return]\n";
+    <STDIN>;
 }
 
-sub progress_step($$) {
-    my $self = shift;
-    my $bin = shift;
 
-    print '.' if($bin);
+sub announce_abi {
+    my $self = shift;
+    my %vars = @_;
+
+    $self->_announce('The currently running kernel has an ABI compatible upgrade pending.', %vars);
 }
 
-sub progress_fin($) {
+
+sub announce_ver {
     my $self = shift;
+    my %vars = @_;
 
-    print "\n";
-
-    $|--;
+    $self->_announce("The currently running kernel version is not the expected kernel version $vars{EVERSION}.", %vars);
 }
 
 
@@ -69,18 +69,27 @@ sub notice($$) {
 
 
 sub _query($$) {
+    my $self = shift;
     my($query, $def) = @_;
-    my @def = ($def eq 'Y' ? qw(yes no) : qw(no yes));
+    my @def = ($def eq 'Y' ? qw(yes no all skip) : qw(no yes all skip));
 
     my $i;
     do {
-	print "$query [", ($def eq 'Y' ? 'Yn' : 'yN'), '] ';;
+	print "$query [", ($def eq 'Y' ? 'Ynas' : 'yNas'), '] ';
+	if($self->{stdio_same}) {
+	    print "$self->{stdio_same}\n";
+	    return $self->{stdio_same};
+	}
+
 	$i = lc(<STDIN>);
 	chomp($i);
 	$i =~ s/^\s+//;
 	$i =~ s/\s+$//;
     } while(!( ($i) = map { (substr($_, 0, length($i)) eq $i ? ($_) : ())} @def ));
 
+    return ($self->{stdio_same} = 'yes') if($i eq 'all');
+    return ($self->{stdio_same} = 'no') if($i eq 'skip');
+
     return $i;
 }
 
@@ -91,13 +100,11 @@ sub query_pkgs($$$$$) {
     my $pkgs = shift;
     my $cb = shift;
 
-    print "$out\n";
-    foreach my $pkg (sort keys %$pkgs) {
-	print "\n$pkg:\n";
+    delete($self->{stdio_same});
 
-	foreach my $rc (keys %{ $pkgs->{$pkg} }) {
-	    &$cb($rc) if(_query("Restart $rc?", ($def ? 'N' : 'Y')) eq 'yes');
-	}
+    print "$out\n";
+    foreach my $rc (sort keys %$pkgs) {
+	&$cb($rc) if($self->_query("Restart $rc?", ($def ? 'N' : 'Y')) eq 'yes');
     }
 }
 
